#!/bin/bash
#
# generate-packages-website.sh -- cron's script to generate packages.medibuntu.org
#
# Copyright © 2008-2009 Lionel Le Folgoc <mrpouit@ubuntu.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

## available releases
RELEASES="precise quantal raring"
## available components
COMPONENTS="free non-free"
## reference arch (first processed)
BASE_ARCH="i386"
## common architectures
ARCHITECTURES="$BASE_ARCH amd64"
## use ARCH_$release for the releases that have more architectures
## string for each distribution (main page)
DISPLAY_precise="Precise (12.04)"
DISPLAY_quantal="Quantal (12.10)"
DISPLAY_raring="Raring (13.04)"
## base URL of the repository (will be used to generate the path to dsc files)
BASE_URL="http://packages.medibuntu.org"
REPO_URL="/srv/medibuntu/repo"
## output
DESTDIR="./www"

# import needed functions
. ./resources/package.sh

function parse_n_build {
	if [ $# -ne 3 ]; then
		echo 'Parses a Packages file to create a page for each binary package found.'
		echo 'Usage: parse_n_build $arch $release $component'
		exit 1
	fi
	# Handle arch: all packages differently
	if [ "$1" = "all" ]; then
		grep-dctrl -n -s Package -F Architecture all Packages-$1 > Available-$1
	else
		grep-dctrl -n -s Package --not -F Architecture all Packages-$1 > Available-$1
	fi
	
	while read line; do
		package=$(echo $line | awk '{print $1}')
		# Get the package section (useful later)
		section=$(grep-dctrl -n -s Section -X -P $package Packages-$1)
		# Get the source package name (useful later too)
		srcname=$(grep-dctrl -n -s Source -X -P $package Packages-$1)
		# If the binary package has the same name as the source package, there is no "Source:" field
		test -z "$srcname" && srcname=$package || true
		# Get the package directory on the server (useful later too)
		dirname=$(grep-dctrl -n -s Directory -X -P $srcname Sources-$3)
		# If the dirname is empty, try again
		if [ -z "$dirname" ]; then
			tmpdirname=$(grep-dctrl -n -s Filename -X -P $package Packages-$1)
			# If this is still empty, give up
			test -n "$tmpdirname" && dirname=$(dirname "$tmpdirname") || dirname="THIS-IS-A-BUG-AND-THIS-SHOULD-NOT-HAPPEN" && true
		fi
		if [ \( "$1" \!= "BASE_ARCH" \) -a \( -f $package.html \) ]; then
			# The page already exists for this package, just append the download link
			append_arch_download $package $1
		else
			# Normal case, just build everything
			build_package_page $package $1 $2 $3 $section $srcname $dirname
		fi
	done < Available-$1
}

echo "$(date -R): started."
# clean
rm -rf $DESTDIR

# let's go
mkdir $DESTDIR && cd $DESTDIR

# Main index page pre-processing
cat ../resources/index_header.html >> index.tmp

for release in $RELEASES; do
	rm -rf $release
	mkdir $release && cd $release
	
	eval release_str=\$DISPLAY_$release

	# check whether the common architectures list is overridden or not
	ARCH_release=ARCH_$release
	test -z "${!ARCH_release}" && eval $ARCH_release=\"$ARCHITECTURES\" || true
	
	for component in $COMPONENTS; do
		cp -a $REPO_URL/dists/$release/$component/source/Sources Sources-$component
		
		for arch in ${!ARCH_release}; do
			cp -a $REPO_URL/dists/$release/$component/binary-$arch/Packages Packages-$arch
			
			# First, $arch-specific
			parse_n_build $arch $release $component
			
			# Cleaning
			rm -f Available-$arch
			if [ "$arch" \!= "$BASE_ARCH" ]; then
				# The Packages-$BASE_ARCH file is used later to process arch: all packages
				rm -f Packages-$arch
			fi
		done
		
		# Handles arch: all packages differently (they are boring)
		mv Packages-$BASE_ARCH Packages-all
		parse_n_build "all" $release $component
		
		# Final cleaning
		rm -f Available-all Packages-all Sources-$component
		
	done
	
	# Main release page post-processing
	sed "s/@RELEASE@/$release/g" ../../resources/release_header.html >> $release.tmp
	# An empty release should not happen, but workaround anyway
	echo "<li style=\"display: none;\"></li>" >> $release.tmp
	
	for page in *.html; do
		# Now we can add the footer safely
		sed "s/@TIMESTAMP@/$(date -R)/" ../../resources/package_footer.html >> $page
		# And create a summary page for $release
		echo "<li><a href=\"$page\">$(basename $page .html)</a></li>" >> $release.tmp
	done
	
	sed "s/@TIMESTAMP@/$(date -R)/" ../../resources/release_footer.html >> $release.tmp
	mv $release.tmp index.html
	
	# do not forget to empty ARCH_release for the next release
	ARCH_release=
	cd ..
	
	# Main index page processing
	echo "<li><a href=\"$release/index.html\">$release_str</a></li>" >> index.tmp
done

# Main index page post-processing
sed "s/@TIMESTAMP@/$(date -R)/" ../resources/index_footer.html >> index.tmp
mv index.tmp index.html

cd ..

# data files
cp -a resources/packages.css $DESTDIR/.
cp -a resources/*.png $DESTDIR/.
cp -a $DESTDIR/* /srv/medibuntu/repo/.
echo "$(date -R): stopped."

exit 0
